#include "box.ch"
#include "inkey.ch"
#include "ooclip.ch"
#include "pd_oop.ch"

CREATE CLASS oMenu 

VAR aChoices
VAR aMainMenu
VAR lType

Var nChoice, nCurChoice

VAR cStyle, cColor, cSide, cShade

Var nRow, nCol
VAR nMenuTop, nMenuLeft, nMenuBottom, nMenuRight

// even though it says it's been fixed to allow any function name...
// i found function name still must be: class_name+function_name.

METHOD New    = oMenuNew,;
       SCalc  = oMenuSCalc

METHOD Read   = oMenuRead,;
       MMenu  = oMenuMMenu,;
       SMenu  = oMenuSMenu,;
       Udf    = oMenuUdf

METHOD Kill   = oMenuKill       

// aadd(handle,{ nposition, 15 } )   // manually save class handle, how many var are in class

ENDCLASS

* --------------------------------------
*   Description: Object Initializer for the window class
*     Arguments: nTop        - the top coordinate of the window
*                nLeft       - the left coordinate of the window
*                nBottom     - the bottom coordinate of the window
*                nRight      - the right coordinate of the window
*                lTmr        - the flag that determines if text mode redefinition is used
*                cStyle      - the 9 character box border/fill style
*                cColor      - the window color
*                cSide       - the side of the window the shadow goes on
*                              "L" - Left
*                              "R" - Right
*                              " " - None
*                cShade      - the shadow shade color, e.g. asc(7)
*       Returns: self
*   Assumptions: NONE
* --------------------------------------
METHOD FUNCTION oMenuNew( aChoices,lType,oObj,;
                          cStyle,cColor,cSide,cShade )

// allow the option to grab information from an existing object.....
::cStyle        := IFNIL(cStyle,if( oObj#NIL,oObj:cStyle,N_SINGLE))
::cColor        := IFNIL(cColor,if( oObj#NIL,oObj:cColor,'b/w,w+/b'  )) 
::cSide         := IFNIL(cSide ,if( oObj#NIL,oObj:cSide ,'R'     )) 
::cShade        := IFNIL(cShade,if( oObj#NIL,oObj:cShade,CHR(8)  ))

// default to top row if not provided by passed object
::nMenuTop      := if( oObj#NIL,oObj:nScrTop,0 )
::nMenuLeft     := if( oObj#NIL,oObj:nScrLeft, 0 )
::nMenuBottom   := if( oObj#NIL,oObj:nScrBottom, 0 )
::nMenuRight    := IF( oObj#NIL,oObj:nScrRight, Maxcol() )

::aMainMenu := {}               // default to empty main menu
::aChoices := IFNIL( achoices, {} )          // load the menu array...
::lType    := IFNIL( lType , .t.         )      // default to main menu

::nRow     := maxrow()/2        // center pop-up menu
::nCol     := maxcol()/2

return self

* --------------------------------------
*   Description: to load the main menu choices
*     Arguments: none
*       Returns: self
*   Assumptions: NONE
* --------------------------------------
METHOD FUNCTION oMenuSCalc
LOCAL X,spot

  // get CHOICE, add sub-menu marker if needed, remove hot-key marker
FOR X = 1 TO LEN(::aChoices)
    spot := strtran( IF( VALTYPE(::aChoices[X])='A',;
                         ::aChoices[X,1],;
                         ::aChoices[X] ),'~','')

    AADD( ::aMainMenu, spot )     // CREATE CHOICE ARRAY
next

return self


* --------------------------------------
*   Description: to initialize main or sub menu
*     Arguments: none
*       Returns: self
*   Assumptions: aScreen has been initialized by begin()/end()
* --------------------------------------
METHOD FUNCTION oMenuRead
local npos,;
      x    := 0,;
      aCol :={},;
      oclr:=setcolor( ::cColor )

if ::lType
   if len(::aChoices)>0 .and. len(::aMainMenu)=0
      ::SCalc()  // build the main menu
   endif
   
   ::MMenu()                                    // do the full main menu
else
   ::nChoice := ::Smenu( ::aChoices,::nRow,::nCol)     // do a box menu
endif

return self


* --------------------------------------
*   Description: to read all main menu choices
*     Arguments: none
*       Returns: self
*   Assumptions: aScreen has been initialized by begin()/end()
* --------------------------------------
METHOD FUNCTION oMenuMMenu
local npos,;
      x    := 0,;
      aCol :={},;
      oclr:=setcolor( ::cColor )

if len(::aChoices)>0 .and. len(::aMainMenu)=0
   ::SCalc()  // build the main menu
endif

do while .t.
dispbegin()
dispbox(::nMenuTop,::nMenuLeft-1,::nMenuTop,::nMenuRight+1,repl(' ',9),::cColor)

npos := 2       // starting menu position
for x = 1 to len( ::aMainMenu )
    @ ::nMenuTop,nPos prompt ::aMainMenu[x]
    aadd( aCol, nPos )                     // save the display position
    nPos += len(::aMainMenu[x])+2
next
dispend()

menu to ::nCurChoice    // save the main menu selection
setcolor( oclr )

if ::nCurChoice > 0 .and. valtype( ::aChoices[::nCurChoice] )=='A' // is there a sub-menu ???
   x := ::SMenu( ::aChoices[::nCurChoice,2], ::nMenuTop+1,aCOL[::nCurChoice] )
endif

if lastkey()=K_LEFT .or. lastkey()=K_RIGHT
   keyboard chr(lastkey())+chr(K_ENTER)
   loop
else
    exit
endif
enddo

::nChoice := ::nCurChoice+(x/100)

return self


* --------------------------------------
*   Description: to read all sub menu choices
*     Arguments: none
*       Returns: self
*   Assumptions: aScreen has been initialized by begin()/end()
* --------------------------------------
METHOD FUNCTION OMenuSMenu( ARAY, ROW,COL )
LOCAL X,spot,o2,;
      ARAY1:={},;
      y  := 0,;
      X2 := 0

  // get CHOICE, add sub-menu marker if needed, remove hot-key marker
FOR X = 1 TO LEN(ARAY)
  spot := strtran( IF( VALTYPE(Aray[X])='A',;
                     Aray[X,1]+' '+chr(16),;
                     Aray[X] ),'~','')

  AADD( ARAY1, spot )     // CREATE CHOICE ARRAY

  IF len(spot)>Y        // FIND THE LONGEST CHOICE
     Y=Len(spot)
  ENDIF
next

Z = MIN( row+LEN(ARAY)+1,MAXROW() )           

dispbegin()
   o2:= oWin():new(row,col,z,col+y+1,,::cStyle,::cColor,::cSide,::cShade)      // ,,B_SINGLE)
   o2:show()
dispend()

oclr:= SETCOLOR(o2:cColor)
   x := achoice( ROW+1,COL+1, Z-1,col+y, aray1,,::Udf )
setcolor( oclr )

IF x > 0 .and. VALTYPE( Aray[X] )='A'
   X2 = ::SMenu( Aray[X,2],ROW(),COL() )  // RECURSE THRU THE SUB MENU ARRAY
ENDIF
o2:Kill()                               // remove the sub-menu from the screen

RETURN X+(X2/100)

**************************************************
METHOD FUNCTION oMenuUdf( modep )
memvar opt_trigr,curr_menu,lastmenu,menu_trigr
local lasTKEY,intitle

LASTKEY = LASTKEY()
do case
   case lastkey = 13         && *- if CR pressed, something has been selected
        return 1
   case lastkey = 27         && *- if ESC was pressed, user wants out
        return 0

        ******** 5-12-95 modified to handle home & end.....
   case lastkey = 1          && home
        keyboard chr( 31 )   && go to top of menu
        return 2
   case lastkey = 6          && end
        keyboard chr( 30 )   && go to bottom of menu
        return 2

        ********
//   case (lastkey > 32 .AND. lastkey < 128)      && *- if we have an ascii character
//        *   case (lastkey > 1 .AND. lastkey < 128)      && *- if we have an ascii character
//
//        IF UPPER(CHR(lastkey)) $ UPPER(opt_trigr)    &&    *- see if its one of our option triggers
//           KEYBOARD CHR(13)  && *- if it is, return 3 (go to next matching element) and keyboard CR to accept
//           return 3
//
//        ELSEIF UPPER(CHR(lastkey)) $ UPPER(menu_trigr)       && *- failing that, see if its one of our menu  triggers
//           intitle = AT(UPPER(CHR(lastkey)),menu_trigr)      && *- find out what menu it is
//           IF !intitle = curr_menu   && *- if its not the current menu, change to it, return an abort to achoice
//              curr_menu = intitle
//              return 0
//           ENDIF
//        ENDIF
endcase

DO CASE                 && *- but wait, there's more...
   CASE modep = 0
        return 2
   CASE modep < 3
        KEYBOARD IIF(modep=1,CHR(30),CHR(31) )
        return 2
   CASE modep = 3 .AND. (lastkey = 4 .OR. lastkey = 19)

//        IF lastkey = K_LEFT          && *- if its the right or left arrows, increment or decrement the menu # and send achoice an abort
//           o3:nCurChoice = if(o3:nCurChoice=1,len(aray),o3:nCurChoice--)
//        ELSE
//           o3:nCurChoice = if(o3:nCurChoice=len(aray),1,o3:nCurChoice++)
//        ENDIF

        return 0
   OTHERWISE
        return 2
ENDCASE

return 2


* --------------------------------------
*   Description: Object Destructor for the menu class
*     Arguments: none
*       Returns: self
*   Assumptions: NONE
* --------------------------------------
METHOD FUNCTION oMenuKill

// reduce all variables

::lType := .t.
::aMainMenu :={}
::aChoices := {}
store ''  to ::cStyle,::cColor,::cSide,::cShade
store 0 to   ::nMenuTop,::nMenuLeft,::nMenuBottom,::nMenuRight,;
             ::nRow,::nCol,;
             ::nChoice,::nCurChoice

return self

                                   