/*Ŀ
ݳ                                                                      
ݳ Program Name: MYALERT.PRG                                            
ݳ     Language: Clipper 5.0                                            
ݳ       Author: John Graceland                                         
ݳ Demo Compile: clipper myalert /n/m/w/dJG_TEST                        
ݳ      Compile: clipper myalert /n/m/w                                 
Ĵ
ݳ Comments:                                                            
ݳ Substitute for Clipper's ALERT()                                     
ݳ Make sure to include code in NewChar.obj if using TMR box type       
Ĵ
ݳ History:                                                             
ݳ KSG 01/94  added SET(_SET_DEVICE,"SCREEN" )                          
ݳ KSG 01/94  added manifest constant to remove demo function           
ݳ KSG 01/94  added ability to use a shadow with John's alert by        
ݳ            defining KG_SHADOW. This shadow uses Clipper's GT api     
ݳ            and is compatible with Blinker v3.nn - protected mode     
ݳ KSG 11/95  excluded shadow in C from current demo since the thin     
ݳ            boxs just added look better w/o shadows                   
ݳ jfw 03/97  modified to fit pd_oop code. please see original code     
ݳ            file PCT_LOOK.zip for great examples                      

           */

#include "inkey.ch"
#include "box.ch"
#include "pd_oop.ch"

// used by SetMode - you could use a SetGet function instead!
#define N_DEF_SCRN_MODE 25

/*
*  Function: Alert( <cMessage> [,<aOptions_>, <nTop>, <cColor>, <lBell>] )
*          :
*   Purpose: Emulates Clippers alert(), additional options listed below
*          :
* Arguments: <cMessage>     - same as Clippers alert
*          : <aOptions_>    - array for buttons, defaults to {" Ok "}
*          : <nTop>         - Starting row position of box
*          : <cColor>       - set color
*          : <lBell>        - sound bell
*          :
*  Comments: Envoke Clippers dbugger from within Alert()
*          : Returns a numeric value equal to the option selected,
*          : <Esc> = 0
*          :
*    Tweaks: KSG 01/94 added
*          : SET(_SET_DEVICE,"SCREEN" )
*          : to insure output always goes to the display screen
*          :
*/

*examples:
*? Alert( "Alert box with no choices")
*? Alert( "Alert box with choices", {"OK","Not OK","Maybe","Never"} )
*? Alert( "Alert box starting on row 16", {"OK","Not OK","Maybe","Never"},16 )
*? Alert( "Multiple Line message:;Message one;Message two;Message three", {"OK","Not OK","Maybe","Never"})
*? Alert( "Alert box with my Color", {"OK","Not OK","Maybe","Never"},,"W+/+BG,N/W")
*? Alert( "Alert box with bell tone", {"OK","Not OK","Maybe","Never"},,"W+/+BG,N/W",.T. )
*? Alert( "Alert box with a shadow",,,,,.T.)

FUNCTION Alert( cMessage, aOptions_, nTop, cColor, lBell, lShadow )

LOCAL o2                    // THE SCREEN OBJECT

local nChoice    := 0
local nVar       := 0
local nCol       := 0
local nRow       := 0
local nPromLeft  := 0
local nBoxLeft   := 0
local nBoxRight  := 0
local nMsgLen    := 0
local nBoxLen    := 0
local nPromLen   := 0

local cOldColor  := SETCOLOR()
local cOldWrap   := SET( _SET_WRAP , "ON" )
local lOldCenter := SET( _SET_MCENTER, .F. )
local lOldBlink  := SETBLINK(.F.)
local aMessage   := {}
local bPgUp      := SETKEY(K_PGUP, { || NIL } )
local bPgDn      := SETKEY(K_PGDN, { || NIL } )
local cDevice    := SET(_SET_DEVICE, "SCREEN" )

lShadow := if(valtype(lShadow) == "L", lShadow,.F.)

if aOptions_ == NIL
   aOptions_ := {}
   AADD( aOptions_," OK ")
endif

// set length of prompt line
AEVAL( aOptions_, { |aArray| nPromLen += LEN(aArray)+2 } )

// load message array
if ";" $ cMessage
   do while ";" $ cMessage      // test for multiple line message
      AADD(aMessage, SUBSTR(cMessage,1,AT(";",cMessage)-1 ) )
      cMessage := STUFF(cMessage,1,AT(";",cMessage), "" )
   enddo
endif

if "" != cMessage       // add remaining message string to array
   AADD(aMessage,cMessage)
endif

nMsgLen := LEN(aMessage)

// determine box length based on largest message line
AEVAL( aMessage, {|aArray| nBoxLen := MAX(nBoxLen,LEN(aArray)+4) })

nPromLen += ( (LEN(aOptions_)+1) * 2 )
cColor   := IF(ISCOLOR(),IF(cColor == NIL,"W+/R,W+/B",cColor),"N/W,W+/N")
lBell    := IF( lBell == NIL, .F., lBell )
nTop     := IF( nTop == NIL, (maxrow()/2)-2, nTop )         // calc center of screen rows

// test for the larger of box lenght vs. length of prompt line
nBoxLen   := IF( nBoxLen > nPromLen, nBoxLen, nPromLen + 4 )

nBoxLeft  := ( MAXCOL() - nBoxLen ) / 2
nBoxRight := nBoxLeft + nBoxLen
nPromLeft := ( ( nBoxLen-nPromLen ) / 2)  // stay inside the box

if procname(1) == "DEFERROR"
   setmode(N_DEF_SCRN_MODE)
endif

DEFINE  o2 FROM  nTop,nBoxLeft TO nTop+3+nMsgLen, nBoxRight+2 ;     // use calc'd positions for display box
TMR   if( procname(1) == "DEFERROR",.f.,.T.)    ;
TITLE '! Alert !'                               ;
COLOR cColor                                    ;
SIDE  if( lShadow, "R", NIL)                    ;
STYLE if( procname(1) == "DEFERROR",B_SINGLE,N_SINGLE )

for nVar := 1 to nMsgLen
   nRow := -1+nVar
   @ nRow, 0 o2:say padc(aMessage[nVar],nBoxLen,SPACE(1))  // CENTERED POSITIONS
next

// calculate the prompt line
nRow += 2
nCol := nPromLeft +3

o2:end()
ACTIVATE o2

if lBell
   TONE(499,2)
   TONE(299,1)
endif

for nVar := 1 to len( aOptions_ )
   @ nRow, nCol o2:PROMPT SPACE(1) + aOptions_[nVar] + SPACE(1)
   nCol += LEN(aOptions_[nVar]) +4
next
MENU TO nChoice

DEACTIVATE o2

// reset environment
SET( _SET_WRAP   , cOldWrap )
SET( _SET_MCENTER, lOldCenter )
SETBLINK( lOldBlink )
SET(_SET_DEVICE, cDevice )

SETCOLOR( cOldColor )

SETKEY(K_PGUP, bPgUp )
SETKEY(K_PGDN, bPgDn )

RETURN IF( ( LASTKEY() == K_ESC  .OR. ;
             LASTKEY() == K_PGUP .OR. ;
             LASTKEY() == K_PGDN    ),;
             0, nChoice )

