#include "inkey.ch"
#include "ooclip.ch"
#include "pd_oop.ch"

CREATE CLASS oScr FROM oWin

VAR aScreen
VAR aCtrlMenu

VAR cTitle
VAR aOldPos

VAR nScrTop
VAR nScrLeft
VAR nScrBottom
VAR nScrRight

// even though it says it's been fixed to allow any function name...
// i found function name still must be: class_name+function_name.

METHOD New    = oScrNew,;
       SCalc  = oScrSCalc

METHOD Begin  = oScrBegin,;
       End    = oScrEnd

METHOD Show   = oScrShow,;
       Read   = oScrRead,;
       Exec   = oScrExec

METHOD Ctrl   = oScrCtrl,;
       CMenu  = oScrCMenu

// aadd(handle,{ nposition, 8 } )   // manually save class handle, how many var are in class

ENDCLASS

* --------------------------------------
*   Description: Object Initializer for the window class
*     Arguments: nTop        - the top coordinate of the window
*                nLeft       - the left coordinate of the window
*                nBottom     - the bottom coordinate of the window
*                nRight      - the right coordinate of the window
*                lTmr        - the flag that determines if text mode redefinition is used
*                cStyle      - the 9 character box border/fill style
*                cColor      - the window color
*                cSide       - the side of the window the shadow goes on
*                              "L" - Left
*                              "R" - Right
*                              " " - None
*                cShade      - the shadow shade color, e.g. asc(7)
*       Returns: self
*   Assumptions: NONE
* --------------------------------------
METHOD FUNCTION oScrNew( nTop, nLeft, nBottom, nRight,;
                         cTitle,lTmr,aCtrlMenu,;
                         cStyle, cColor, cSide, cShade )

::oWinNew( nTop, nLeft, nBottom, nRight,;
           lTmr, cStyle, cColor, cSide, cShade)
::cTitle    := ifNil(cTitle,'')
::aCtrlMenu := ifNil(aCtrlmenu,{})
::SCalc()

return self

* --------------------------------------
*   Description: to calculate the available screen positions
*     Arguments: none
*       Returns: self
*   Assumptions: NONE
* --------------------------------------
METHOD FUNCTION oScrSCalc
local nVal := if( empty(::cStyle) .or. ::cStyle==NIL, 0, 1 ) // if no border, use full screen positioning

::nScrTop   := ::nTop    + nVal   // calc screen available for display
::nScrLeft  := ::nLeft   + nVal   // to be 1 character less than screen box
::nScrBottom:= ::nBottom - nVal
::nScrRight := ::nRight  - nVal

::aOldPos   := { ::nScrTop,::nScrLeft,::nScrBottom,::nScrRight }

return self


* --------------------------------------
*   Description: to add a say object to the screen object
*     Arguments: none
*       Returns: self
*   Assumptions: NONE
* --------------------------------------
METHOD FUNCTION oScrBegin
local cScreen := savescreen(::nScrTop,::nScrLeft,::nScrBottom,::nScrRight)
clear gets
dispbegin()

::oWinshow():Ctrl()                            // display the window (hidden)
setcolor( ::cColor )                           // set to default window color

::aScreen := { cScreen, getlist }
return self

* --------------------------------------
*   Description: to complete a say objects to the screen object
*     Arguments: none
*       Returns: self
*   Assumptions: NONE
* --------------------------------------   
METHOD FUNCTION oScrEnd
local cScreen := savescreen(::nScrTop,::nScrLeft,::nScrBottom,::nScrRight)
local aGets1  := getlist

clear gets
restscreen(::nScrTop,::nScrLeft,::nScrBottom,::nScrRight,::aScreen[1])
getlist := ::aScreen[2]

::hide()                                       // remove the screen object from display

dispend()
::aScreen := { cScreen, aGets1 }

return self

* --------------------------------------
*   Description: to display all say objects to the screen
*     Arguments: none
*       Returns: self
*   Assumptions: aScreen has been initialized by begin()/end()
* --------------------------------------
METHOD FUNCTION oScrShow

::oWinshow():Ctrl()                            // display the box
setcolor( ::cColor )                           // set to default window color

restscreen(::nScrTop,::nScrLeft,::nScrBottom,::nScrRight,::aScreen[1])

return self


* --------------------------------------
*   Description: to read all get objects to the screen
*     Arguments: none
*       Returns: self
*   Assumptions: aScreen has been initialized by begin()/end()
* --------------------------------------
METHOD FUNCTION oScrRead

getlist := ::aScreen[2]
read
return self


* --------------------------------------
*   Description: to display all say/get objects to the screen object
*     Arguments: none  (this is a shell to other methods)
*       Returns: self
*   Assumptions: NONE
* --------------------------------------
METHOD FUNCTION oScrExec

::show()
::read()
set key K_ALT_F10 to 

return self

* --------------------------------------
*  Description: to add control menu options to the screen object
*     Arguments: none
*       Returns: self
*   Assumptions: NONE
* --------------------------------------
METHOD FUNCTION oScrCtrl
local bt_esc := if( ::lTmr, B_ESC, '[=]'           )
local bt_up  := if( ::lTmr, B_UP ,     chr( 30)+']')
local bt_dn  := if( ::lTmr, B_DN , '['+chr( 31)    )

if !empty(::cTitle)
   @ ::nTop,::nLeft say padc( ::cTitle,(::nRight-::nLeft)+1) color 'n/gr'
endif
@ ::nTop,::nLeft say bt_esc             color 'w/n'

if len(::aCtrlMenu)>0 .and. ascan( ::aCtrlMenu, "MAX" )>0
   @ ::nTop,::nRight-1 say Bt_UP        color 'w/n'

   if ascan( ::aCtrlMenu, "MIN" )>0                     // if there is a down button to be displayed
      @ ::nTop,::nRight-3 say Bt_DN     color 'w/n'
   elseif !::lTmr                                       // else end the non-tmr button display
      @ ::nTop,::nRight-2 say '['       color 'w/n'
   endif
endif
   
if ascan( ::aCtrlMenu, "EXIT" )=0
   aadd( ::aCtrlMenu,"EXIT" )
endif

set key K_ALT_F10 to ::CMenu()

return self

* --------------------------------------
*   Description: to display the control menu to the screen object
*     Arguments: none
*       Returns: self
*   Assumptions: NONE
* --------------------------------------
METHOD FUNCTION oScrCMenu
local x,cScreen,lScrChange:=.f.

cScreen:= savescreen(::nScrTop,::nLeft,::nScrTop+len(::aCtrlMenu)+1,::nLeft+15)

dispbox(::nScrTop,::nLeft,::nScrTop+len(::aCtrlMenu)+1,::nLeft+15,::cStyle)

x      := achoice(::nScrTop+1,::nLeft+1,::nScrTop+len(::aCtrlMenu)+1,::nLeft+14,;
                  ::aCtrlMenu)
restscreen(::nScrTop,::nLeft,::nScrTop+len(::aCtrlMenu)+1,::nLeft+15,cScreen)

do case
*   case ::aCtrlMenu[x]="EXIT"
*        keyboard K_PGDN
*        return self
   case ::aCtrlMenu[x]="MAXIMIZE"
        ::nTop    := 0
        ::nLeft   := 0
        ::nBottom := maxrow()
        ::nRight  := maxcol() 
        lScrChange := .t.
    
   case ::aCtrlMenu[x]="MINIMIZE"
        ::nTop    := maxrow()-1
        ::nLeft   := 0
        ::nBottom := maxrow()
        ::nRight  := 10
        lScrChange := .t.
endcase

if lScrChange                           // did we change the size of the box ???
   ::SCalc()
   ::Hide()
   ::Exec()
endif

keyboard K_PGDN

if lScrChange
   keyboard K_TAB
endif

return self
