#include "box.ch"
#include "ooclip.ch"
#include "pd_oop.ch"

* --------------------------------------
*   Description: Window Class definition
*       Returns: CLASS
* --------------------------------------
CLASS oWin

var nTop, nLeft, nBottom, nRight
var cSide, cShade, cStyle, cColor
var nRow, nCol, ;
    oldrow, oldcol, ;
    nForrow, nForcol
var lTmr
var nStop, nSleft, nSbottom, nSright

var cBackScreen, cForeScreen

var cShadSide, cShadChar, cBoxStyle, cBoxColor

*************************

method New           = oWinNew,;
       setcoords     = oWinsetcoords,;
       SetStyle      = oWinSetStyle

       method Show          = oWinShow,;
       SaveBackground= oWinSaveBackground,;
       ShowShadow    = oWinShowShadow,;
       SetShadow     = oWinSetShadow

method Hide          = oWinHide,;
       SaveForeground= oWinSaveForeground,;
       PopForeground = oWinPopForeground

method Kill          = oWinKill

// aadd(handle,{ nposition, 25 } )   // manually save class handle, how many var are in class

endclass


* --------------------------------------
*   Description: Object Initializer for the window class
*     Arguments: nTop        - the top coordinate of the window
*                nLeft       - the left coordinate of the window
*                nBottom     - the bottom coordinate of the window
*                nRight      - the right coordinate of the window
*                cStyle      - the 9 character box border/fill style
*                cColor      - the window color
*                cSide       - the side of the window the shadow goes on
*                              "L" - Left
*                              "R" - Right
*                              " " - None
*                cShade      - the shadow shade color, e.g. asc(7)
*       Returns: self
*   Assumptions: NONE
* --------------------------------------
METHOD FUNCTION oWinNew( nTop, nLeft, nBottom, nRight,;
                         lTmr,;
                         cStyle, cColor, cSide, cShade)

::lTmr  := IFNIL( lTmr, .t. )                     // allow true
::setcoords( nTop, nLeft, nBottom, nRight )       // set box area
::SetStyle( cStyle, cColor, cSide, cShade )       // set box style

return self

* --------------------------------------
*   Description: to set the window coordinate iVars.
*     Arguments:
*       Returns: self
*   Assumptions: NONE
* --------------------------------------
METHOD FUNCTION oWinsetcoords(nTop, nLeft, nBottom, nRight)

::nTop    := IFNIL( nTop   , 0 )          // default to full screen
::nLeft   := IFNIL( nLeft  , 0 )
::nBottom := IFNIL( nBottom, MAXROW() )
::nRight  := IFNIL( nRight , MAXCOL() )

::nrow    := ::nTop
::ncol    := ::nLeft
::nSTop := ::nSLeft := ::nSBottom := ::nSRight := 0  // default to no shadow

return self

* --------------------------------------
*   Description: to set the window style iVars.
*     Arguments:
*       Returns: self
*   Assumptions: NONE
* --------------------------------------
METHOD FUNCTION oWinSetStyle( cStyle, cColor, cSide, cShade )

::cBoxStyle := if(::lTmr,N_SINGLE,B_DOUBLE+' ')      // default styles
::cBoxColor := "W+/B"
::cShadSide  := "R"
::cShadChar := CHR(8)

::cStyle        := IfNil( cStyle , ::cBoxStyle)
::cColor        := IfNil( cColor , ::cBoxColor)
::cSide         := IfNil( cSide  , ::cShadSide )
::cShade        := IfNil( cShade , ::cShadChar)

::cSide         := upper( ::cSide )

if ::lTmr                 // are we using text mode redefinition ??
   kg_init()
endif

return self

* --------------------------------------
*   Description: Method to draw the window on the screen.
*     Arguments: NONE
*       Returns: self
*   Assumptions: NONE
* --------------------------------------
METHOD FUNCTION oWinShow()

if ::cSide $ "RL"         // calc extra shadow positions
   ::nSTop     := ::nTop+1    // default to right
   ::nSBottom  := ::nBottom+1
   ::nSLeft    := ::nLeft+1
   ::nSRight   := ::nRight+1
   if ::cSide = "L"
      ::nSLeft  -= 2
      ::nSRight -= 2
   endif
endif

dispbegin()                  // no paint

::oldRow := row()      // save old screen and position
::oldCol := col()
::SaveBackground()        // save original screen data

dispbox( ::nTop, ::nLeft, ::nBottom, ::nRight, ;
         ::cStyle, ::cColor)            // do the box

if ::cSide $ "RL"
   ::ShowShadow()                         // do the shadow
endif

DISPEND()                    // pop it all up at once

return self

* --------------------------------------
*   Description: to save the screen background under the window
*     Arguments: NONE
*       Returns: self
*   Assumptions: NONE
* --------------------------------------
METHOD FUNCTION oWinSaveBackground()

// save the largest possible used area
::cBackScreen := SAVESCREEN( ::nTop,     MIN( ::nSLeft ,::nLeft ),;
::nSBottom, MAX( ::nSRight,::nRight) )
return self

* --------------------------------------
*   Description: to display the window objects shadow.
*     Arguments: NONE
*       Returns: self
*   Assumptions: NONE
* --------------------------------------
METHOD FUNCTION oWinShowShadow()

local nSedge := if( ::cSide = "L",::nSleft,::nSright) // which side ??

if ::nSBottom <= MAXROW()              // do the bottom shadow ???
   restscreen( ::nSbottom,::nSleft,::nSbottom,::nSright, ;
               ::setshadow(savescreen( ::nSbottom,::nSleft,::nSbottom,::nSright) )) // bottom
ENDIF

if (::cSide = "L" .and. nSedge >= 0) .or.;
   (::cSide = "R" .and. nSedge <= maxcol() )  // do the side shadow ???
   restscreen( ::nStop,nSedge,::nSbottom,nSedge, ;
               ::setshadow(savescreen( ::nStop,nSedge,::nSbottom,nSedge) )) // side
ENDIF

return self

* --------------------------------------
*   Description: Creates the window objects shadow.
*     Arguments: NONE
*       Returns: dimmed screen display as string
*   Assumptions: NONE
* --------------------------------------
METHOD FUNCTION oWinSetShadow( Str )

local x, r_str := ""                            // x=counter, r_str=return_string

for x = 1 to len( str ) step 2                    // every other char is color
    r_str += substr(str,x,1) + ::cShade       // change it...
next

return r_str


* --------------------------------------
*   Description: hide the window object by restoring its background
*     Arguments: NONE
*       Returns: self
*   Assumptions: NONE
* --------------------------------------
METHOD FUNCTION oWinHide()

::SaveForeground()      // save current window contents for possible restore

RESTSCREEN( ::nTop,    MIN( ::nSLeft, ::nLeft ),;
            ::nSBottom,MAX( ::nSRight,::nRight),;
            ::cBackScreen )                     // restore the old screen

DEVPOS(::oldRow, ::oldCol)          // restore the old screen position

return self

* --------------------------------------
*   Description: to save the screen contents inside the window
*     Arguments: NONE
*       Returns: self
*   Assumptions: NONE
* --------------------------------------
METHOD FUNCTION oWinSaveForeground()

::nForrow     := row()
::nForcol     := col()
::cForeScreen := SAVESCREEN( ::nTop+1,::nLeft+1,::nBottom-1,::nRight-1)

return self


* --------------------------------------
*   Description: to restore the screen contents to the window
*     Arguments: NONE
*       Returns: self
*   Assumptions: NONE
* --------------------------------------
METHOD FUNCTION oWinPOPForeground()

// restore hidden screen contents & cursor position
RESTSCREEN( ::nTop+1,::nLeft+1,::nBottom-1,::nRight-1,;
            ::cForeScreen )

devpos( ::nForrow , ::nForcol )

return self


* --------------------------------------
*   Description: Method to recover the window object for memory
*     Arguments: NONE
*       Returns: self
*   Assumptions: NONE
* --------------------------------------
METHOD FUNCTION oWinKill()

::Hide()             // restore the old screen & minimize object variables

::nTop:=::nLeft:=::nBottom:=::nRight:=0
::nRow:=::nCol:=::oldrow:=::oldcol:=nForrow:=nForcol:=0

::cBackScreen:=cForeScreen:=''
::cSide:=::cShade:=::cStyle:=::cColor:=''
::cShadSide:=::cShadChar:=::cBoxStyle:=::cBoxColor:=''

return self
           